@@ -3,7 +3,6 @@
 
 #include "r_types.h"
 #include "r_util.h"
-#include "list.h" // TODO: port to r_list
 
 #ifdef __cplusplus
 extern "C" {
@@ -34,13 +33,15 @@ typedef struct r_config_node_t {
 	char *desc;
 } RConfigNode;
 
+R_API const char *r_config_node_type(RConfigNode *node);
+
 typedef struct r_config_t {
 	int lock;
 	int last_notfound;
 	int n_nodes;
 	void *user;
 	RNum *num;
-	PrintfCallback printf;
+	PrintfCallback cb_printf;
 	RList *nodes;
 	RHashTable *ht;
 } RConfig;
@@ -51,6 +52,7 @@ R_API RConfig *r_config_clone (RConfig *cfg);
 R_API int r_config_free(RConfig *cfg);
 R_API void r_config_lock(RConfig *cfg, int l);
 R_API int r_config_eval(RConfig *cfg, const char *str);
+R_API void r_config_bump(RConfig *cfg, const char *key);
 R_API RConfigNode *r_config_set_i(RConfig *cfg, const char *name, const ut64 i);
 R_API RConfigNode *r_config_set_cb(RConfig *cfg, const char *name, const char *value, int (*callback)(void *user, void *data));
 R_API RConfigNode *r_config_set_i_cb(RConfig *cfg, const char *name, int ivalue, int (*callback)(void *user, void *data));
@@ -64,9 +66,13 @@ R_API void r_config_list(RConfig *cfg, const char *str, int rad);
 R_API RConfigNode *r_config_node_get(RConfig *cfg, const char *name);
 R_API RConfigNode *r_config_node_new(const char *name, const char *value);
 R_API void r_config_node_free(void *n);
-R_API int r_config_swap(RConfig *cfg, const char *name);
+R_API int r_config_toggle(RConfig *cfg, const char *name);
 R_API int r_config_readonly (RConfig *cfg, const char *key);
 
+/*----------------------------------------------------------------------------------------------*/
+R_API void r_config_set_sort_column (char *column);
+/*----------------------------------------------------------------------------------------------*/
+
 R_API int r_config_set_setter (RConfig *cfg, const char *key, RConfigCallback cb);
 R_API int r_config_set_getter (RConfig *cfg, const char *key, RConfigCallback cb);
 #endif
